package xyz.thastertyn.UserInterface;

import java.io.IOException;
import java.util.concurrent.atomic.AtomicBoolean;

import com.googlecode.lanterna.TerminalPosition;
import com.googlecode.lanterna.TerminalSize;
import com.googlecode.lanterna.gui2.Borders;
import com.googlecode.lanterna.gui2.Direction;
import com.googlecode.lanterna.gui2.Label;
import com.googlecode.lanterna.gui2.LinearLayout;
import com.googlecode.lanterna.gui2.Panel;
import com.googlecode.lanterna.gui2.Window;
import com.googlecode.lanterna.gui2.WindowBasedTextGUI;
import com.googlecode.lanterna.gui2.WindowListener;
import com.googlecode.lanterna.gui2.dialogs.MessageDialog;
import com.googlecode.lanterna.gui2.dialogs.MessageDialogButton;
import com.googlecode.lanterna.input.KeyStroke;
import com.googlecode.lanterna.input.KeyType;

import xyz.thastertyn.UserInterface.Content.JecnaContent;
import xyz.thastertyn.UserInterface.Content.OmluvnyList;
import xyz.thastertyn.UserInterface.Content.Rozvrh;
import xyz.thastertyn.UserInterface.Content.Sdeleni;
import xyz.thastertyn.UserInterface.Content.Znamky;
import xyz.thastertyn.UserInterface.Dialogs.EscapeDialog;

public class WindowSwitchListener implements WindowListener, UpdateListener {
	private WindowBasedTextGUI textGUI;

	private JecnaContent[] contents = {
		new Znamky(this),
		new Rozvrh(this),
		new Sdeleni(this),
		new OmluvnyList(this)
	};

	private Label[] tabs = new Label[contents.length];

	private Panel tabsPanel = new Panel().setLayoutManager(new LinearLayout(Direction.HORIZONTAL));
	private Panel holderPanel = new Panel();

	int current = contents.length - 1;

	public WindowSwitchListener(Panel holder, WindowBasedTextGUI textGUI)
	{
		this.textGUI = textGUI;
		holder.addComponent(tabsPanel);
		holder.addComponent(holderPanel.withBorder(Borders.singleLine()));
		
		for(int i = 0; i < contents.length; i++)
		{
			this.tabs[i] = contents[i].getLabel();
			tabsPanel.addComponent(this.tabs[i]);
		}

		next();
	}

	private void next()
	{
		if(current + 1 == contents.length)
		{
			current = 0;
		}else{
			current++;
		}
		
		loadPanel();
	}

	private void previous()
	{
		if(current - 1 == -1)
		{
			current = contents.length - 1;
		}else{
			current--;
		}
		loadPanel();
	}

	private void loadPanel()
	{
		checkIfDownloaded();

		holderPanel.removeAllComponents();
		tabsPanel.removeAllComponents();

		setPanelPreferedSize();
		updateLabels();
	}

	private void checkIfDownloaded()
	{
		if(!contents[current].hasStarted())
		{
			try{
				contents[current].downloadDefault();
			}catch(IOException e)
			{
				MessageDialog.showMessageDialog(textGUI, "Something failed", "The content failed to download", MessageDialogButton.OK);
			}
		}
	}

	private void setPanelPreferedSize()
	{
		int tabColumns = tabsPanel
		.getSize()
		.getColumns();

		int currentSize = contents[current]
		.getPanel()
		.getSize()
		.getColumns();
		
		if(currentSize < tabColumns)
		{
			holderPanel.addComponent(
				contents[current].getPanel().setPreferredSize(new TerminalSize(tabColumns, 
				contents[current].getPanel().getPreferredSize().getRows())));
			
		}else{
			holderPanel.addComponent(contents[current].getPanel());
		}
	}

	private void updateLabels()
	{
		for(int i = 0; i < tabs.length; i++)
		{
			if(i == current)
			{
				tabsPanel.addComponent(tabs[i].withBorder(Borders.doubleLine()));
			}else{
				tabsPanel.addComponent(tabs[i].withBorder(Borders.singleLine()));
			}
		}
	}

	private void showOptions()
	{
		try{
			contents[current].showOptions(textGUI);
		}catch(IOException e)
		{
			MessageDialog.showMessageDialog(textGUI, "Something failed", "An error with connection has occured, either no connection at all, or connection timed out", MessageDialogButton.OK);
		}
	}

	@Override
	public void onInput(Window basePane, KeyStroke keyStroke, AtomicBoolean deliverEvent) {
		KeyType type = keyStroke.getKeyType();

		switch(type)
		{
			case Tab:
				if(keyStroke.isShiftDown())
				{
					previous();
				}else{
					next();
				}
				break;
			case Character:
				if(keyStroke.getCharacter() == ' ')
				{
					showOptions();
				}
				break;
			case Escape:
				EscapeDialog dialog = new EscapeDialog();
				dialog.showDialog(textGUI);
				break;
			default:
				break;
		}
	}

	@Override
	public void onUnhandledInput(Window basePane, KeyStroke keyStroke, AtomicBoolean hasBeenHandled) {
		// TODO Auto-generated method stub
	}

	@Override
	public void onResized(Window window, TerminalSize oldSize, TerminalSize newSize) {
		window.invalidate();
	}

	@Override
	public void onMoved(Window window, TerminalPosition oldPosition, TerminalPosition newPosition) {
		// TODO Auto-generated method stub
	}

	@Override
	public void updatePanel() {
		holderPanel.removeAllComponents();
		holderPanel.addComponent(contents[current].getPanel());
	}
	
}
